@@ -4,7 +4,7 @@
  * Description:		This module contains routines related to
  *			delay load import libraries.
  *			
- * Comments:		See "notice.txt" for copyright and license information.
+ * Comments:		See "readme.txt" for copyright and license information.
  *-------
  */
 
@@ -63,6 +63,11 @@
 #if defined(DYNAMIC_LOAD)
 #define	WIN_DYN_LOAD
 CSTR	libpqdll = "LIBPQ.dll";
+#ifdef	_WIN64
+CSTR	gssapidll = "GSSAPI64.dll";
+#else
+CSTR	gssapidll = "GSSAPI32.dll";
+#endif /* _WIN64 */
 #ifdef	UNICODE_SUPPORT
 CSTR	pgenlist = "pgenlist";
 CSTR	pgenlistdll = "PGENLIST.dll";
@@ -76,15 +81,22 @@ CSTR	pgenlistdll = "PGENLISTA.dll";
 #endif /* DYNAMIC_LOAD */
 #endif /* WIN32 */
 
-CSTR	libpq = "libpq";
+CSTR	libpqlib = "libpq";
+#ifdef	_WIN64
+CSTR	gssapilib = "gssapi64";
+#else
+CSTR	gssapilib = "gssapi32";
+#endif /* _WIN64 */
 #ifndef	NOT_USE_LIBPQ
-CSTR	checkproc = "PQconninfoParse";
+CSTR	checkproc1 = "PQconnectdbParams";
+static int	connect_withparam_available = -1;
+CSTR	checkproc2 = "PQconninfoParse";
 static int	sslverify_available = -1;
 #endif /* NOT_USE_LIBPQ */
 
 #if defined(_MSC_DELAY_LOAD_IMPORT)
 static BOOL	loaded_libpq = FALSE, loaded_ssllib = FALSE;
-static BOOL	loaded_pgenlist = FALSE;
+static BOOL	loaded_pgenlist = FALSE, loaded_gssapi = FALSE;
 /*
  *	Load psqlodbc path based libpq dll.
  */
@@ -141,16 +153,24 @@ DliErrorHook(unsigned	dliNotify,
 #else
 			__pfnDliNotifyHook2 = NULL;
 #endif /* _MSC_VER */
-			if (_strnicmp(pdli->szDll, libpq, 5) == 0)
+			if (_strnicmp(pdli->szDll, libpqlib, strlen(libpqlib)) == 0)
 			{
-				if (hmodule = MODULE_load_from_psqlodbc_path(libpq), NULL == hmodule)
-					hmodule = LoadLibrary(libpq);
+				if (hmodule = MODULE_load_from_psqlodbc_path(libpqlib), NULL == hmodule)
+					hmodule = LoadLibrary(libpqlib);
 #ifndef	NOT_USE_LIBPQ
+				if (NULL == hmodule)
+					connect_withparam_available = sslverify_available = FALSE;
+				if (connect_withparam_available < 0)
+				{
+					if (NULL == GetProcAddress(hmodule, checkproc1))
+						connect_withparam_available = FALSE;
+					else
+						connect_withparam_available = TRUE;
+inolog("connect_withparam_available=%d\n", connect_withparam_available); 
+				}
 				if (sslverify_available < 0)
 				{
-					if (NULL == hmodule)
-						sslverify_available = FALSE;
-					else if (NULL == GetProcAddress(hmodule, checkproc))
+					if (NULL == GetProcAddress(hmodule, checkproc2))
 						sslverify_available = FALSE;
 					else
 						sslverify_available = TRUE;
@@ -162,6 +182,23 @@ DliErrorHook(unsigned	dliNotify,
 				if (hmodule = MODULE_load_from_psqlodbc_path(pgenlist), NULL == hmodule)
 					hmodule = LoadLibrary(pgenlist);
 			}
+#ifdef	USE_GSS
+			else if (_strnicmp(pdli->szDll, gssapilib, strlen(gssapilib)) == 0)
+			{
+#ifndef	NOT_USE_LIBPQ
+                		if (hmodule = GetModuleHandle(gssapilib), NULL == hmodule)
+#endif
+				{
+					if (hmodule = MODULE_load_from_psqlodbc_path(gssapilib), NULL == hmodule)
+					{
+						if (hmodule = LoadLibrary(gssapilib), NULL != hmodule)
+							loaded_gssapi = TRUE;
+					}
+					else
+						loaded_gssapi = TRUE;
+				}
+			}
+#endif /* USE_GSS */
 			else if (0 == _stricmp(pdli->szDll, libarray[0]) ||
 				 0 == _stricmp(pdli->szDll, libarray[1]))
 			{
@@ -223,6 +260,11 @@ void CleanupDelayLoadedDLLs(void)
 		success = (*func)(pgenlistdll);
 		mylog("%s unload success=%d\n", pgenlistdll, success);
 	}
+	if (loaded_gssapi)
+	{
+		success = (*func)(gssapidll);
+		mylog("%s unload success=%d\n", gssapidll, success);
+	}
 	return;
 }
 #else
@@ -267,12 +309,12 @@ BOOL	ssl_verify_available(void)
 			sslverify_available = 0;
 #else
 #ifdef HAVE_LIBLTDL
-		lt_dlhandle dlhandle = lt_dlopenext(libpq);
+		lt_dlhandle dlhandle = lt_dlopenext(libpqlib);
 
 		sslverify_available = 1;
 		if (NULL != dlhandle)
 		{
-			if (NULL == lt_dlsym(dlhandle, checkproc))
+			if (NULL == lt_dlsym(dlhandle, checkproc2))
 				sslverify_available = 0;
 			lt_dlclose(dlhandle);
 		}
@@ -283,6 +325,46 @@ BOOL	ssl_verify_available(void)
 	return (0 != sslverify_available);
 }
 
+BOOL	connect_with_param_available(void)
+{
+	if (connect_withparam_available < 0)
+	{
+#if defined(_MSC_DELAY_LOAD_IMPORT)
+		__try {
+#if (_MSC_VER < 1300)
+			__pfnDliFailureHook = DliErrorHook;
+			__pfnDliNotifyHook = DliErrorHook;
+#else
+			__pfnDliFailureHook2 = DliErrorHook;
+			__pfnDliNotifyHook2 = DliErrorHook;
+#endif /* _MSC_VER */
+			PQescapeLiteral(NULL, NULL, 0);
+		}
+		__except (filter_env2encoding(GetExceptionCode())) {
+		}
+		if (connect_withparam_available < 0)
+{
+inolog("connect_withparam_available is set to false\n"); 
+			connect_withparam_available = 0;
+}
+#else
+#ifdef HAVE_LIBLTDL
+		lt_dlhandle dlhandle = lt_dlopenext(libpqlib);
+
+		connect_withparam_available = 1;
+		if (NULL != dlhandle)
+		{
+			if (NULL == lt_dlsym(dlhandle, checkproc1))
+				connect_withparam_available = 0;
+			lt_dlclose(dlhandle);
+		}
+#endif /* HAVE_LIBLTDL */
+#endif /* _MSC_DELAY_LOAD_IMPORT */
+	}
+
+	return (0 != connect_withparam_available);
+}
+
 void *CALL_PQconnectdb(const char *conninfo, BOOL *libpqLoaded)
 {
 	void *pqconn = NULL;
@@ -296,6 +378,7 @@ void *CALL_PQconnectdb(const char *conninfo, BOOL *libpqLoaded)
 		__pfnDliFailureHook2 = DliErrorHook;
 		__pfnDliNotifyHook2 = DliErrorHook;
 #endif /* _MSC_VER */
+inolog("calling PQconnectdb\n");
 		pqconn = PQconnectdb(conninfo);
 	}
 	__except ((GetExceptionCode() & 0xffff) == ERROR_MOD_NOT_FOUND ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH) {
@@ -319,11 +402,53 @@ void *CALL_PQconnectdb(const char *conninfo, BOOL *libpqLoaded)
 #endif /* _MSC_DELAY_LOAD_IMPORT */
 	return pqconn;
 }
+
+void *CALL_PQconnectdbParams(const char *opts[], const char *vals[], BOOL *libpqLoaded)
+{
+	void *pqconn = NULL;
+	*libpqLoaded = TRUE;
+#if defined(_MSC_DELAY_LOAD_IMPORT)
+	__try {
+#if (_MSC_VER < 1300)
+		__pfnDliFailureHook = DliErrorHook;
+		__pfnDliNotifyHook = DliErrorHook;
+#else
+		__pfnDliFailureHook2 = DliErrorHook;
+		__pfnDliNotifyHook2 = DliErrorHook;
+#endif /* _MSC_VER */
+inolog("calling PQconnectdbParams\n");
+		pqconn = PQconnectdbParams(opts, vals, 0);
+	}
+	__except ((GetExceptionCode() & 0xffff) == ERROR_MOD_NOT_FOUND ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH) {
+		*libpqLoaded = FALSE;
+	}
+#if (_MSC_VER < 1300)
+	__pfnDliNotifyHook = NULL;
+#else
+	__pfnDliNotifyHook2 = NULL;
+#endif /* _MSC_VER */
+	if (*libpqLoaded)
+	{
+		loaded_libpq = TRUE;
+		/* ssllibs are already loaded by libpq
+		if (PQgetssl(pqconn))
+			loaded_ssllib = TRUE;
+		*/
+	}
+#else
+	pqconn = PQconnectdbParams(opts, vals, 0);
+#endif /* _MSC_DELAY_LOAD_IMPORT */
+	return pqconn;
+}
 #else
 BOOL	ssl_verify_available(void)
 {
 	return	FALSE;
 }
+BOOL	connect_with_param_available(void)
+{
+	return	FALSE;
+}
 #endif /* NOT_USE_LIBPQ */
 
 #ifdef	_HANDLE_ENLIST_IN_DTC_
@@ -382,9 +507,9 @@ BOOL SSLLIB_check()
 	mylog("checking libpq library\n");
 	/* First search the driver's folder */
 #ifndef	NOT_USE_LIBPQ
-	if (NULL == (hmodule = MODULE_load_from_psqlodbc_path(libpq)))
+	if (NULL == (hmodule = MODULE_load_from_psqlodbc_path(libpqlib)))
 		/* Second try the PATH ordinarily */
-		hmodule = LoadLibrary(libpq);
+		hmodule = LoadLibrary(libpqlib);
 	mylog("libpq hmodule=%p\n", hmodule);
 #endif /* NOT_USE_LIBPQ */
 #ifdef	USE_SSPI
