@@ -115,8 +115,8 @@ UCHAR *check_client_encoding(const UCHAR *conn_settings)
 	int	step = 0;
 	size_t	len = 0;
 
-        for (cptr = conn_settings; *cptr; cptr++)
-        {
+	for (cptr = conn_settings; *cptr; cptr++)
+	{
 		if (in_quote)
 			if (LITERAL_QUOTE == *cptr)
 			{
@@ -521,6 +521,8 @@ const char * get_environment_encoding(const ConnectionClass *conn, const char *s
 #endif /* UNICODE_SUPPORT */
 	if (setenc && stricmp(setenc, OTHER_STRING))
 		return setenc;
+	if (wenc = getenv("PGCLIENTENCODING"), NULL != wenc)
+		return wenc;
 #ifdef	WIN32
 	acp = GetACP();
 	if (acp >= 1251 && acp <= 1258)
