@@ -9,7 +9,7 @@
  * API functions:	SQLSetConnectOption, SQLSetStmtOption, SQLGetConnectOption,
  *					SQLGetStmtOption
  *
- * Comments:		See "notice.txt" for copyright and license information.
+ * Comments:		See "readme.txt" for copyright and license information.
  *--------
  */
 
@@ -261,13 +261,13 @@ set_statement_option(ConnectionClass *conn,
 				if (stmt)
 				{
 					SC_set_error(stmt, STMT_NOT_IMPLEMENTED_ERROR, "Unknown statement option (Set)", NULL);
-					sprintf(option, "fOption=%d, vParam=" FORMAT_LEN, fOption, vParam);
+					sprintf(option, "fOption=%d, vParam=" FORMAT_ULEN, fOption, vParam);
 					SC_log_error(func, option, stmt);
 				}
 				if (conn)
 				{
 					CC_set_error(conn, CONN_NOT_IMPLEMENTED_ERROR, "Unknown statement option (Set)", func);
-					sprintf(option, "fOption=%d, vParam=" FORMAT_LEN, fOption, vParam);
+					sprintf(option, "fOption=%d, vParam=" FORMAT_ULEN, fOption, vParam);
 					CC_log_error(func, option, conn);
 				}
 
@@ -734,7 +734,7 @@ PGAPI_GetStmtOption(
 			{
 				/* make sure we're positioned on a valid row */
 				if ((ridx < 0) ||
-					(ridx >= QR_get_num_cached_tuples(res)))
+					(((SQLULEN) ridx) >= QR_get_num_cached_tuples(res)))
 				{
 					SC_set_error(stmt, STMT_INVALID_CURSOR_STATE_ERROR, "Not positioned on a valid row.", func);
 					return SQL_ERROR;
