@@ -3,10 +3,7 @@
  * Description:		This file contains defines and declarations that are related to
  *					the entire driver.
  *
- * Comments:		See "notice.txt" for copyright and license information.
- *
- * $Id: psqlodbc.h,v 1.135 2011/06/19 12:27:30 hinoue Exp $
- *
+ * Comments:		See "readme.txt" for copyright and license information.
  */
 
 #ifndef __PSQLODBC_H__
@@ -148,10 +145,10 @@ typedef	unsigned long long ULONG_PTR;
 #else
 #error appropriate long pointer type not found 
 #endif /* SIZEOF_VOID_P */
-#if (SIZEOF_VOID_P == 8)
+#if (SIZEOF_LONG_INT == 8)
 #define	FORMAT_INTEGER	"%d"	/* SQLINTEGER */
 #define	FORMAT_UINTEGER	"%u"	/* SQLUINTEGER */
-#if defined(WITH_UNIXODBC) && !defined(BUILD_REAL_64_BIT_MODE)
+#if defined(WITH_UNIXODBC) && defined(BUILD_LEGACY_64_BIT_MODE)
 #define FORMAT_LEN	"%d"	/* SQLLEN */
 #define FORMAT_ULEN	"%u"	/* SQLULEN */
 #else
